import logging


class Warnings(object):
    def __init__(self, logger=None):
        self.warnings = list()
        self.logger = logger or logging

    def warning(self, message):
        self.warnings.append(message)
        self.logger.warning(message)

    def __bool__(self):
        if self.warnings:
            return True
        else:
            return False


def decrypt_TradeInfo_TradeSha(
    HashKey,
    HashIV,
    TradeInfo,
    TradeSha=None,
    use_json=True,
):
    from Crypto.Cipher import AES
    from Crypto.Hash import SHA256
    from urllib.parse import parse_qs
    import codecs
    import json

    def removepadding(message, blocksize=32):
        if message[-1] < blocksize:
            return message[:(-1 * ord(message[-1:])):]
        return message

    aes_obj = AES.new(HashKey, AES.MODE_CBC, HashIV)

    if TradeSha:
        hash = SHA256.new()
        hash.update('HashKey={}&{}&HashIV={}'.format(HashKey, TradeInfo, HashIV).encode())
        TradeSha_verify = hash.hexdigest().upper()
        if TradeSha != TradeSha_verify:
            raise Exception('TradeSha not match')

    TradeInfo_decrypted = removepadding(aes_obj.decrypt(codecs.decode(TradeInfo, 'hex_codec'))).decode()

    if use_json:
        TradeInfo_dict = json.loads(TradeInfo_decrypted)
    else:
        TradeInfo_dict = parse_qs(TradeInfo_decrypted)

    return TradeInfo_dict


def generate_TradeInfo_TradeSha(
    HashKey,                # HashKey
    HashIV,                 # HashIV
    MerchantID,             # 商店代號　　　　　　    智付通商店代號
                            ##################################################################################################
    MerchantOrderNo,        # 商店訂單編號　　　　    商店自訂訂單編號，限英、數字、”_ ” 格式。
    Amt,                    # 訂單金額　　　　　　    純數字不含符號 幣別:新台幣
    ItemDesc,               # 商品資訊　　　　　　    限制長度為 50 字
    Email=None,             # 付款人電子信箱
    LoginType=False,        # 智付通會員　　　　　    1=須要登入智付通會員 0=不須登入智付通會員
                            ##################################################################################################
    RespondType='JSON',     # 回傳格式　　　　　　    JSON 或是 String
    TimeStamp=None,         # 時間戳記　　　　　　    自從 Unix 纪元
    Version='1.4',          # 串接程式版本
    LangType='zh-tw',       # 語系　　　　　　　　    英文版參數為 en 繁體中文版參數為 zh-tw
                            # 　　　　　　　　　　    當未提供此參數或此參數數值錯誤時，將預設為繁體中文版
                            ##################################################################################################
    EmailModify=None,       # 不須登入智付通會員　    1=可修改(預設) 0=不可修改
    OrderComment=None,      # 商店備註　　　　　　    限制長度為 300 字 若有提供此參數，將會於 MPG 頁面呈現 商店備註內容
    TradeLimit=None,        # 交易限制秒數　　　　    秒數下限為 60 秒 上限為 900 秒 若未帶此參數，或是為 0 時，會視作為不啟用交易限制秒數。
    ExpireDate=None,        # 繳費有效期限　　　　    (適用於非即時交易) 格式為 date('Ymd') ，例:20140620
                            # 　　　　　　　　　　    此參數若為空值，系統預設為 7 天。自取 號時間起算至第 7 天 23:59:59。
                            ##################################################################################################
    ReturnURL=None,         # 支付完成返回商店網址    若為空值，交易完成後，消費者將停留在智付通付款或取號完成頁面
    NotifyURL=None,         # 支付通知網址
    CustomerURL=None,       # 商店取號網址　　　　    此參數若為空值，則會顯示取號結果在智付通頁面。
    ClientBackURL=None,     # 支付取消返回商店網址    此參數若為空值時，則無返回鈕。
                            ##################################################################################################
    CREDIT=None,            # 信用卡一次付清啟用　    設定是否啟用信用卡一次付清支付方式 1=啟用 0或者未有此參數=不啟用
    InstFlag=None,          # 信用卡分期付款啟用　    此欄位值=1 時，即代表開啟所有分期期別
                            # 　　　　　　　　　　    同時開啟多期別時，將此參數用","分隔，例如:3,6,12，代表開啟分 3、6、12期的功能
                            # 　　　　　　　　　　    此欄位值=0或無值時，即代表不開啟分期
    CreditRed=None,         # 信用卡紅利啟用　　　    1=啟用 0或者未有此參數=不啟用
    UNIONPAY=None,          # 信用卡銀聯卡啟用　　    1=啟用 0或者未有此參數=不啟用
    WEBATM=None,            # ＷＥＢＡＴＭ啟用　　    1=啟用 0或者未有此參數=不啟用
    VACC=None,              # ＡＴＭ轉帳啟用　　　    1=啟用 0或者未有此參數=不啟用
    CVS=None,               # 超商代碼繳費啟用　　    1=啟用 0或者未有此參數=不啟用 訂單金額小於30元或超過2萬元仍不會顯示此支付方式
    BARCODE=None,           # 超商條碼繳費啟用　　    1=啟用 0或者未有此參數=不啟用 訂單金額小於20元或超過4萬元仍不會顯示此支付方式
                            ##################################################################################################
    order_list=None,
    request=None,
    *args,
    **kwargs
):
    from Crypto.Cipher import AES
    from Crypto.Hash import SHA256
    from email.utils import parseaddr
    from urllib.parse import urlencode
    import datetime
    import re
    import time

    warnings = Warnings()

    def addpadding(message, blocksize=32):
        pad = blocksize - (len(message) % blocksize)
        return message + chr(pad) * pad

    if not isinstance(MerchantID, str):
        warnings.warning('MerchantID is not string')
        MerchantID = str(MerchantID)

    if not isinstance(MerchantOrderNo, str):
        MerchantOrderNo = str(MerchantOrderNo)
    if not re.match('^[A-Za-z0-9_]*$', MerchantOrderNo):
        warnings.warning('MerchantOrderNo only accept string contains only letters, numbers, and underscores. Bypass then')

    if not isinstance(Amt, int):
        warnings.warning('Amt is not integer')
        Amt = int(Amt)

    if not isinstance(ItemDesc, str):
        ItemDesc = str(ItemDesc)
        if len(ItemDesc) > 50:
            warnings.warning('Length of ItemDesc should be less than 50. Bypass then')

    if Email is not None:
        parsed_email = parseaddr(Email)[-1]
        if '@' in parsed_email and parsed_email != Email:
            warnings.warning('Email is not legal. Bypass then')
    else:
        warnings.warning('Email is not optional field. Bypass then')

    if LoginType is not None:
        LoginType = 1 if LoginType else 0
    else:
        warnings.warning('LoginType is not optional field. Bypass then')

    if RespondType not in ('JSON', 'String'):
        warnings.warning('RespondType should be \'JSON\' or \'String\'')

    if TimeStamp is None:
        TimeStamp = int(time.time())
    elif isinstance(TimeStamp, datetime.datetime):
        TimeStamp = int(TimeStamp.timestamp())
    elif isinstance(TimeStamp, int) or isinstance(TimeStamp, float):
        TimeStamp = int(datetime.datetime.fromtimestamp(TimeStamp).timestamp())
    elif isinstance(TimeStamp, str):
        warnings.warning('Unable to parse TimeStamp by . Bypass then')
    else:
        warnings.warning('TimeStamp not str, time, or datetime. Bypass then.')

    if Version != '1.4':
        warnings.warning('Version is not current version, which is 1.4')

    if LangType:
        if LangType not in ('en', 'zh-tw'):
            warnings.warning('LangType is not set to en or zh-tw. Service provider may use zh-tw instead. By pass then.')

    if TradeLimit is not None:
        if isinstance(TradeLimit, int):
            warnings.warning('TradeLimit is not integer. By pass then.')
        elif TradeLimit < 60:
            warnings.warning('TradeLimit is less than 60. Service provider may use 60 instead. By pass then.')
        elif TradeLimit > 900:
            warnings.warning('TradeLimit is greater than 900. Service provider may use 900 instead. By pass then.')

    if ExpireDate is not None:
        if isinstance(ExpireDate, datetime.datetime) or isinstance(ExpireDate, time):
            ExpireDate = ExpireDate.strftime('%Y%m%d')

    if EmailModify is not None:
        EmailModify = 1 if EmailModify else 0
    if OrderComment is not None:
        OrderComment = str(OrderComment) if OrderComment else None
    if CREDIT is not None:
        CREDIT = 1 if CREDIT else 0

    if InstFlag is not None:
        if InstFlag is True:
            InstFlag = '1'
        elif InstFlag is False:
            InstFlag = '0'
        elif isinstance(InstFlag, list):
            InstFlag = ','.join(InstFlag)

    if CreditRed is not None:
        CreditRed = 1 if CreditRed else 0
    if UNIONPAY is not None:
        UNIONPAY = 1 if UNIONPAY else 0
    if WEBATM is not None:
        WEBATM = 1 if WEBATM else 0
    if VACC is not None:
        VACC = 1 if VACC else 0
    if CVS is not None:
        CVS = 1 if CVS else 0
        if CVS:
            if Amt < 30 or Amt > 20000:
                warnings.warning('Service provider may not display CVS due to Amt is not between 30 and 20000')
    if BARCODE is not None:
        BARCODE = 1 if BARCODE else 0
        if BARCODE:
            if Amt < 20 or Amt > 40000:
                warnings.warning('Service provider may not display BARCODE due to Amt is not between 20 and 40000')

    if request:
        if ReturnURL and not ReturnURL.startswith('http://') and not ReturnURL.startswith('https://'):
            ReturnURL = request.build_absolute_uri(str(ReturnURL))
        if NotifyURL and not NotifyURL.startswith('http://') and not NotifyURL.startswith('https://'):
            NotifyURL = request.build_absolute_uri(str(NotifyURL))
        if CustomerURL and not CustomerURL.startswith('http://') and not CustomerURL.startswith('https://'):
            CustomerURL = request.build_absolute_uri(str(CustomerURL))
        if ClientBackURL and not ClientBackURL.startswith('http://') and not ClientBackURL.startswith('https://'):
            ClientBackURL = request.build_absolute_uri(str(ClientBackURL))

    if order_list is None:
        order_list = list()

    original_order_list = (
        'MerchantID',
        'RespondType',
        'TimeStamp',
        'Version',
        'LangType',
        'MerchantOrderNo',
        'Amt',
        'ItemDesc',
        'TradeLimit',
        'ExpireDate',
        'ReturnURL',
        'NotifyURL',
        'CustomerURL',
        'ClientBackURL',
        'Email',
        'EmailModify',
        'LoginType',
        'OrderComment',
        'CREDIT',
        'InstFlag',
        'CreditRed',
        'UNIONPAY',
        'WEBATM',
        'VACC',
        'CVS',
        'BARCODE',
    )

    aes_obj = AES.new(HashKey, AES.MODE_CBC, HashIV)
    message = dict(
        MerchantID=MerchantID,
        MerchantOrderNo=MerchantOrderNo,
        Amt=Amt,
        ItemDesc=ItemDesc,
        Email=Email,
        RespondType=RespondType,
        TimeStamp=TimeStamp,
        Version=Version,
        LangType=LangType,
        TradeLimit=TradeLimit,
        ExpireDate=ExpireDate,
        ReturnURL=ReturnURL,
        NotifyURL=NotifyURL,
        CustomerURL=CustomerURL,
        ClientBackURL=ClientBackURL,
        EmailModify=EmailModify,
        LoginType=LoginType,
        OrderComment=OrderComment,
        CREDIT=CREDIT,
        InstFlag=InstFlag,
        CreditRed=CreditRed,
        UNIONPAY=UNIONPAY,
        WEBATM=WEBATM,
        VACC=VACC,
        CVS=CVS,
        BARCODE=BARCODE,
    )

    message_list = list()

    for each_key in order_list:
        value = message.get(each_key)
        if value is not None:
            message_list.append((each_key, value))
    for each_key in original_order_list:
        value = message.get(each_key)
        if each_key not in order_list and value is not None:
            message_list.append((each_key, value))

    from collections import OrderedDict

    ordered_message = OrderedDict(message_list)

    message_str = addpadding(urlencode(ordered_message))
    TradeInfo = aes_obj.encrypt(message_str).hex()

    hash = SHA256.new()
    hash.update('HashKey={}&{}&HashIV={}'.format(HashKey, TradeInfo, HashIV).encode())
    TradeSha = hash.hexdigest().upper()

    return TradeInfo, TradeSha, warnings



from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
from spgateway.models import generate_slug


def generate_slug_for_none(apps, schema_editor):
    order_appname = settings.SPGATEWAY_ORDERMODEL.split('.', maxsplit=1)[0]
    order_classname = settings.SPGATEWAY_ORDERMODEL.split('.', maxsplit=1)[1]
    Order = apps.get_model(order_appname, order_classname)
    for each in Order.objects.filter(SpgatewaySlug=None):
        if not each.SpgatewaySlug:
            each.SpgatewaySlug = generate_slug()
            each.save()


class Migration(migrations.Migration):
    dependencies = [
        ('spgateway', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(generate_slug_for_none),
    ]



import time

from django.conf import settings
from django.urls import reverse_lazy
from django.utils.http import http_date

SPGATEWAY_DEFAULT_SAMESITE_COOKIE_NAME = 'spgateway_sessionid'


def spgateway_session_key_plain_encrypt_decrypt_function(session_key):
    return session_key


class SpgatewaySameSiteCookieMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        self.return_url_set = set()
        self.customer_url_set = set()
        self.encrypt_function = getattr(
            settings, 'SPGATEWAY_ANTISAMESITE_ENCRYPT_FUNCTION',
            spgateway_session_key_plain_encrypt_decrypt_function,
        )
        self.decrypt_function = getattr(
            settings, 'SPGATEWAY_ANTISAMESITE_DECRYPT_FUNCTION',
            spgateway_session_key_plain_encrypt_decrypt_function,
        )

        for shop_id, shop in settings.SPGATEWAY_PROFILE.items():
            return_url = shop.get('ReturnURL')
            if return_url:
                self.return_url_set.add(return_url)

            customer_url = shop.get('CustomerURL')
            if customer_url:
                self.customer_url_set.add(customer_url)

        if not self.return_url_set:
            self.return_url_set.add(reverse_lazy('spgateway_ReturnView'))

        if not self.customer_url_set:
            self.customer_url_set.add(reverse_lazy('spgateway_CustomerView'))

    def __call__(self, request):

        response = self.get_response(request)

        if hasattr(settings, 'SESSION_COOKIE_SAMESITE') and settings.SESSION_COOKIE_SAMESITE:
            if request.path in self.return_url_set or request.path in self.customer_url_set:
                antisamesite_session_cookie_name = getattr(
                    settings, 'SPGATEWAY_ANTISAMESITE_COOKIE_NAME',
                    SPGATEWAY_DEFAULT_SAMESITE_COOKIE_NAME,
                )
                encrypted_session_key = request.COOKIES.get(antisamesite_session_cookie_name)
                if encrypted_session_key:
                    session_key = self.decrypt_function(encrypted_session_key)
                    max_age = request.session.get_expiry_age()
                    expires_time = time.time() + max_age
                    expires = http_date(expires_time)

                    response.set_cookie(
                        settings.SESSION_COOKIE_NAME,
                        session_key, max_age=max_age,
                        expires=expires, domain=settings.SESSION_COOKIE_DOMAIN,
                        path=settings.SESSION_COOKIE_PATH,
                        secure=settings.SESSION_COOKIE_SECURE or None,
                        httponly=settings.SESSION_COOKIE_HTTPONLY or None,
                        samesite=settings.SESSION_COOKIE_SAMESITE,
                    )
                    response.delete_cookie(antisamesite_session_cookie_name)
            elif request.session.session_key:
                encrypted_session_key = self.encrypt_function(request.session.session_key)

                response.set_cookie(
                    key=getattr(settings, 'SPGATEWAY_ANTISAMSITE_COOKIE_NAME', SPGATEWAY_DEFAULT_SAMESITE_COOKIE_NAME),
                    value=encrypted_session_key
                )

        return response



from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^return/$', views.SpgatewayReturnView.as_view(), name='spgateway_ReturnView'),
    url(r'^customer/$', views.SpgatewayCustomerView.as_view(), name='spgateway_CustomerView'),
    url(r'^notify/$', views.SpgatewayNotifyView.as_view(), name='spgateway_NotifyView'),
]



from django.conf import settings
from django.db import models
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse

import uuid

from .helpers import generate_TradeInfo_TradeSha
from .forms import SpgatewayForm


# Create your models here.
class SpgatewayResponseMixin(object):
    Status = models.CharField(max_length=10, verbose_name='回傳狀態')
    Message = models.CharField(max_length=50, verbose_name='回傳訊息')
    Result = models.TextField(verbose_name='回傳參數')


class SpgatewayNotifyResponseInfo(SpgatewayResponseMixin, models.Model):
    Order = models.ForeignKey(settings.SPGATEWAY_ORDERMODEL, verbose_name='訂單', on_delete=models.CASCADE)
    MerchantID = models.CharField(max_length=15, verbose_name='商店代號')
    Amt = models.IntegerField(verbose_name='交易金額')
    TradeNo = models.CharField(max_length=20, verbose_name='智付通交易序號')
    MerchantOrderNo = models.CharField(max_length=20, verbose_name='商店訂單編號')
    PaymentType = models.CharField(max_length=10, verbose_name='支付方式')
    RespondType = models.CharField(max_length=10, verbose_name='回傳格式')
    PayTime = models.DateTimeField(verbose_name='支付完成時間')
    IP = models.CharField(max_length=15, verbose_name='交易 IP')
    EscrowBank = models.CharField(max_length=10, verbose_name='履保銀行')


class SpgatewayNotifyResponseCredit(models.Model):
    Info = models.ForeignKey(SpgatewayNotifyResponseInfo, on_delete=models.CASCADE)
    RespondCode = models.CharField(max_length=5, verbose_name='金融機構回應碼')
    Auth = models.CharField(max_length=6, verbose_name='授權碼')
    Card6No = models.CharField(max_length=6, verbose_name='卡號前六碼')
    Card4No = models.CharField(max_length=4, verbose_name='卡號末四碼')
    Inst = models.IntegerField(verbose_name='分期-期別')
    InstFirst = models.IntegerField(verbose_name='分期-首期金額')
    InstEach = models.IntegerField(verbose_name='分期-每期金額')
    ECI = models.CharField(max_length=2, verbose_name='ECI 值')
    TokenUseStatus = models.IntegerField(verbose_name='信用卡快速結帳使用狀態')
    RedAmt = models.IntegerField(null=True, verbose_name='紅利折抵後實際金額')


class SpgatewayNotifyResponseATM(models.Model):
    Info = models.ForeignKey(SpgatewayNotifyResponseInfo, on_delete=models.CASCADE)
    PayBankCode = models.CharField(max_length=10, verbose_name='付款人金融機構代碼')
    PayerAccount5Code = models.CharField(max_length=5, verbose_name='付款人金融機構帳號末五碼')


class SpgatewayNotifyResponseCVS(models.Model):
    Info = models.ForeignKey(SpgatewayNotifyResponseInfo, on_delete=models.CASCADE)
    CodeNo = models.CharField(max_length=30, verbose_name='繳費代碼')


class SpgatewayNotifyResponseBarcode(models.Model):
    Info = models.ForeignKey(SpgatewayNotifyResponseInfo, on_delete=models.CASCADE)
    Barcode_1 = models.CharField(max_length=20, verbose_name='第一段條碼')
    Barcode_2 = models.CharField(max_length=20, verbose_name='第二段條碼')
    Barcode_3 = models.CharField(max_length=20, verbose_name='第三段條碼')
    PayStore = models.CharField(max_length=8, verbose_name='繳費超商')


class SpgatewayCustomerResponseInfo(SpgatewayResponseMixin, models.Model):
    Order = models.ForeignKey(settings.SPGATEWAY_ORDERMODEL, verbose_name='訂單', on_delete=models.CASCADE)
    MerchantID = models.CharField(max_length=15, verbose_name='商店代號')
    Amt = models.IntegerField(verbose_name='交易金額')
    TradeNo = models.CharField(max_length=20, verbose_name='智付通交易序號')
    MerchantOrderNo = models.CharField(max_length=20, verbose_name='商店訂單編號')
    PaymentType = models.CharField(max_length=10, verbose_name='支付方式')
    ExpireDate = models.DateTimeField(verbose_name='繳費截止日期')


class SpgatewayCustomerResponseVACC(models.Model):
    Info = models.ForeignKey(SpgatewayCustomerResponseInfo, on_delete=models.CASCADE)
    BankCode = models.CharField(max_length=10, verbose_name='金融機構代碼')
    CodeNo = models.CharField(max_length=30, verbose_name='繳費代碼')


class SpgatewayCustomerResponseCVS(models.Model):
    Info = models.ForeignKey(SpgatewayCustomerResponseInfo, on_delete=models.CASCADE)
    CodeNo = models.CharField(max_length=30, verbose_name='繳費代碼')


class SpgatewayCustomerResponseBarcode(models.Model):
    Info = models.ForeignKey(SpgatewayCustomerResponseInfo, on_delete=models.CASCADE)
    Barcode_1 = models.CharField(max_length=20, verbose_name='第一段條碼')
    Barcode_2 = models.CharField(max_length=20, verbose_name='第二段條碼')
    Barcode_3 = models.CharField(max_length=20, verbose_name='第三段條碼')


def generate_slug():
    hex = uuid.uuid4().hex
    hex = hex.replace('-', '')
    return hex[:20]


class SpgatewayOrderMixin(models.Model):
    SpgatewaySlug = models.SlugField(max_length=20, null=True, unique=True)

    SpgatewayMerchantID = None
    SpgatewayAmt = None
    SpgatewayItemDesc = None
    SpgatewayEmail = None
    SpgatewayClientBackURL = None
    SpgatewayNotifyURL = None
    SpgatewayReturnURL = None
    SpgatewayCustomerURL = None
    SpgatewayLoginType = None
    SpgatewayInstFlag = None
    SpgatewayCreditRed = None

    SpgatewayMerchantIDFieldName = None
    SpgatewayAmtFieldName = None
    SpgatewayItemDescFieldName = None
    SpgatewayEmailFieldName = None
    SpgatewayClientBackURLFieldName = None
    SpgatewayNotifyURLFieldName = None
    SpgatewayReturnURLFieldName = None
    SpgatewayCustomerURLFieldName = None
    SpgatewayLoginTypeFieldName = None
    SpgatewayInstFlagFieldName = None
    SpgatewayCreditRedFieldName = None

    class Meta:
        abstract = True

    def regenerate_slug(self):
        self.SpgatewaySlug = generate_slug()
        self.save()

    def save(self, **kwargs):
        if self.SpgatewaySlug in (None, ''):
            self.SpgatewaySlug = generate_slug()
        super(SpgatewayOrderMixin, self).save(**kwargs)

    def get_SpgatewayMerchantID(self, **kwargs):
        if 'MerchantID' in kwargs.keys():
            return kwargs['MerchantID']
        if 'MerchantIDFieldName' in kwargs.keys():
            return getattr(self, kwargs['MerchantIDFieldName'])
        if self.SpgatewayMerchantID is not None:
            return self.SpgatewayMerchantID
        if self.SpgatewayMerchantIDFieldName is not None:
            return getattr(self, self.SpgatewayMerchantIDFieldName)
        if 'MerchantProfile' in kwargs.keys():
            if 'MerchantID' in kwargs['MerchantProfile'].keys():
                return kwargs['MerchantProfile']['MerchantID']
            if 'MerchantIDFieldName' in kwargs['MerchantProfile'].keys():
                return getattr(self, kwargs['MerchantProfile']['MerchantIDFieldName'])

    def get_SpgatewayAmt(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'Amt' in each_profile.keys():
                return each_profile['Amt']
            if 'AmtFieldName' in each_profile.keys():
                return getattr(self, each_profile['AmtFieldName'])
        if self.SpgatewayAmt is not None:
            return self.SpgatewayAmt
        if self.SpgatewayAmtFieldName is not None:
            return getattr(self, self.SpgatewayAmtFieldName)
        raise ImproperlyConfigured('SpgatewayAmtFieldName not offered')

    def get_SpgatewayItemDesc(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'ItemDesc' in each_profile.keys():
                return each_profile['ItemDesc']
            if 'ItemDescFieldName' in each_profile.keys():
                return getattr(self, each_profile['ItemDescFieldName'])
        if self.SpgatewayItemDesc is not None:
            return self.SpgatewayItemDesc
        if self.SpgatewayItemDescFieldName is not None:
            return getattr(self, self.SpgatewayItemDescFieldName)
        raise ImproperlyConfigured('SpgatewayItemDescFieldName not offered')

    def get_SpgatewayEmail(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'Email' in each_profile.keys():
                return each_profile['Email']
            if 'EmailFieldName' in each_profile.keys():
                return getattr(self, each_profile['EmailFieldName'])
        if self.SpgatewayEmail is not None:
            return self.SpgatewayEmail
        if self.SpgatewayEmailFieldName is not None:
            return getattr(self, self.SpgatewayEmailFieldName)
        raise ImproperlyConfigured('SpgatewayEmailFieldName not offered')

    def get_SpgatewayClientBackURL(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'ClientBackURL' in each_profile.keys():
                return each_profile['ClientBackURL']
            if 'ClientBackURLFieldName' in each_profile.keys():
                return getattr(self, each_profile['ClientBackURLFieldName'])
        if self.SpgatewayClientBackURL is not None:
            return self.SpgatewayClientBackURL
        if self.SpgatewayClientBackURLFieldName is not None:
            return getattr(self, self.SpgatewayClientBackURLFieldName)
        if hasattr(self, 'get_absolute_url'):
            return self.get_absolute_url()
        return None

    def get_SpgatewayNotifyURL(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'NotifyURL' in each_profile.keys():
                return each_profile['NotifyURL']
            if 'NotifyURLFieldName' in each_profile.keys():
                return getattr(self, each_profile['NotifyURLFieldName'])
        if self.SpgatewayNotifyURL is not None:
            return self.SpgatewayNotifyURL
        if self.SpgatewayNotifyURLFieldName is not None:
            return getattr(self, self.SpgatewayNotifyURLFieldName)

        return reverse('spgateway_NotifyView')

    def get_SpgatewayReturnURL(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'ReturnURL' in each_profile.keys():
                return each_profile['ReturnURL']
            if 'ReturnURLFieldName' in each_profile.keys():
                return getattr(self, each_profile['ReturnURLFieldName'])
        if self.SpgatewayReturnURL is not None:
            return self.SpgatewayReturnURL
        if self.SpgatewayReturnURLFieldName is not None:
            return getattr(self, self.SpgatewayReturnURLFieldName)
        return reverse('spgateway_ReturnView')

    def get_SpgatewayCustomerURL(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'CustomerURL' in each_profile.keys():
                return each_profile['CustomerURL']
            if 'CustomerURLFieldName' in each_profile.keys():
                return getattr(self, each_profile['CustomerURLFieldName'])
        if self.SpgatewayCustomerURL is not None:
            return self.SpgatewayCustomerURL
        if self.SpgatewayCustomerURLFieldName is not None:
            return getattr(self, self.SpgatewayCustomerURLFieldName)
        return reverse('spgateway_CustomerView')

    def get_SpgatewayLoginType(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'LoginType' in each_profile.keys():
                return each_profile['LoginType']
            if 'LoginTypeFieldName' in each_profile.keys():
                return getattr(self, each_profile['LoginTypeFieldName'])
        if self.SpgatewayLoginType is not None:
            return self.SpgatewayLoginType
        if self.SpgatewayLoginTypeFieldName is not None:
            return getattr(self, self.SpgatewayLoginTypeFieldName)
        return None

    def get_SpgatewayInstFlag(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'InstFlag' in each_profile.keys():
                return each_profile['InstFlag']
            if 'InstFlagFieldName' in each_profile.keys():
                return getattr(self, each_profile['InstFlagFieldName'])
        if self.SpgatewayInstFlag is not None:
            return self.SpgatewayInstFlag
        if self.SpgatewayInstFlagFieldName is not None:
            return getattr(self, self.SpgatewayInstFlagFieldName)
        return None

    def get_SpgatewayCreditRed(self, **kwargs):
        for each_profile in (kwargs, kwargs.get('MerchantProfile', dict())):
            if 'CreditRed' in each_profile.keys():
                return each_profile['CreditRed']
            if 'CreditRedFieldName' in each_profile.keys():
                return getattr(self, each_profile['CreditRedFieldName'])
        if self.SpgatewayCreditRed is not None:
            return self.SpgatewayCreditRed
        if self.SpgatewayCreditRedFieldName is not None:
            return getattr(self, self.SpgatewayCreditRedFieldName)
        return None

    def _generate_new_kwargs(self, **kwargs):
        merchant_id = kwargs.get('MerchantID', settings.SPGATEWAY_MERCHANTID)
        merchant_profile = settings.SPGATEWAY_PROFILE[merchant_id]

        new_kwargs = dict(
            MerchantID=merchant_id,
            MerchantProfile=merchant_profile,
        )
        new_kwargs.update(kwargs)
        return new_kwargs

    def generate_form(self, **kwargs):
        new_kwargs = self._generate_new_kwargs(**kwargs)

        default_kwargs = dict(
            MerchantID=new_kwargs['MerchantID'],
            HashIV=new_kwargs['MerchantProfile']['HashIV'],
            HashKey=new_kwargs['MerchantProfile']['HashKey'],
            Amt=self.get_SpgatewayAmt(**new_kwargs),
            ItemDesc=self.get_SpgatewayItemDesc(**new_kwargs),
            Email=self.get_SpgatewayEmail(**new_kwargs),
            LoginType=self.get_SpgatewayLoginType(**new_kwargs),
            ClientBackURL=self.get_SpgatewayClientBackURL(**new_kwargs),
            MerchantOrderNo=self.SpgatewaySlug,
            NotifyURL=self.get_SpgatewayNotifyURL(**new_kwargs),
        )
        tradeinfo_kwargs = default_kwargs.copy()
        tradeinfo_kwargs.update(kwargs)

        TradeInfo, TradeSha, warnings = generate_TradeInfo_TradeSha(**tradeinfo_kwargs)

        form_initial_dict = dict(
            MerchantID=new_kwargs['MerchantID'],
            TradeInfo=TradeInfo,
            TradeSha=TradeSha,
        )

        return SpgatewayForm(initial=form_initial_dict)

    def generate_credit_form(self, **kwargs):
        kwargs = self._generate_new_kwargs(**kwargs)

        default_kwargs = dict(
            CREDIT=True,
            InstFlag=self.get_SpgatewayInstFlag(**kwargs),
            CreditRed=self.get_SpgatewayCreditRed(**kwargs),
            ReturnURL=self.get_SpgatewayReturnURL(**kwargs),
        )
        new_kwargs = default_kwargs.copy()
        new_kwargs.update(kwargs)
        return self.generate_form(**new_kwargs)

    def generate_webatm_form(self, **kwargs):
        kwargs = self._generate_new_kwargs(**kwargs)

        default_kwargs = dict(
            WEBATM=True,
            ReturnURL=self.get_SpgatewayReturnURL(**kwargs),
        )
        new_kwargs = default_kwargs.copy()
        new_kwargs.update(kwargs)
        return self.generate_form(**new_kwargs)

    def generate_cvs_form(self, **kwargs):
        kwargs = self._generate_new_kwargs(**kwargs)

        default_kwargs = dict(
            CVS=True,
            CustomerURL=self.get_SpgatewayCustomerURL(**kwargs),
        )
        new_kwargs = default_kwargs.copy()
        new_kwargs.update(kwargs)
        return self.generate_form(**new_kwargs)

    def generate_vacc_form(self, **kwargs):
        kwargs = self._generate_new_kwargs(**kwargs)

        default_kwargs = dict(
            VACC=True,
            CustomerURL=self.get_SpgatewayCustomerURL(**kwargs),
        )
        new_kwargs = default_kwargs.copy()
        new_kwargs.update(kwargs)
        return self.generate_form(**new_kwargs)

    def generate_barcode_form(self, **kwargs):
        kwargs = self._generate_new_kwargs(**kwargs)

        default_kwargs = dict(
            BARCODE=True,
            CustomerURL=self.get_SpgatewayCustomerURL(**kwargs),
        )
        new_kwargs = default_kwargs.copy()
        new_kwargs.update(kwargs)
        return self.generate_form(**new_kwargs)



from django import forms
from django.conf import settings

def get_action_url(**kwargs):
    action_url = getattr(settings, 'SPGATEWAY_MPG_ACTION', '')
    if not action_url:
        merchant_id = kwargs.get('MerchantID', settings.SPGATEWAY_MERCHANTID)
        merchant_profile = settings.SPGATEWAY_PROFILE[merchant_id]
        action_url = merchant_profile.get('MPG_ACTION')
    if not action_url:
        debug = merchant_profile.get('DEBUG', getattr(settings, 'DEBUG', ''))
        if debug:
            action_url = 'https://ccore.spgateway.com/MPG/mpg_gateway'
        else:
            action_url = 'https://core.spgateway.com/MPG/mpg_gateway'
    return action_url


class SpgatewayForm(forms.Form):
    MerchantID = forms.CharField(max_length=15, widget=forms.HiddenInput())
    TradeInfo = forms.CharField(widget=forms.HiddenInput())
    TradeSha = forms.CharField(widget=forms.HiddenInput())
    Version = forms.CharField(max_length=5, initial='1.4', widget=forms.HiddenInput())
    action = get_action_url()



from setuptools import setup

__version__ = "0.7.5"

with open('README.rst') as f:
    long_description = f.read()

setup(
    name="django-spgateway",
    version=__version__,
    description='Django support for Spgateway',
    keywords="django, spgateway",
    author="CJLTSOD <github.tsod@tsod.idv.tw>",
    author_email="github.tsod@tsod.idv.tw",
    url="https://github.com/cjltsod/django-spgateway",
    license="MIT",
    packages=["spgateway"],
    include_package_data=True,
    install_requires=["django>=1.10", "pycrypto>=2.6.1"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Framework :: Django :: 2.1",
        "License :: OSI Approved :: MIT License",
        "Environment :: Web Environment",
    ],
    long_description=long_description,
)
